import 'dart:convert' as convert;

import 'package:http/http.dart';
import 'package:spacex_api/models/capsule.dart';
import 'package:spacex_api/models/company/company.dart';
import 'package:spacex_api/models/pagenated_response.dart';
import 'package:spacex_api/models/query/options.dart';
import 'package:spacex_api/spacex_api.dart';

main(List<String> args) {
  // _fetchSingleCapsule();
  // _fetchAllCapsules();
  _queryCapsules();
  // _fetchCapsules<Capsule>();
  // _fetchCompany<Company>();
}

Future<void> _queryCapsules<T>() async {
  final api = SpaceXApi();
  // final json = {
  //   "options": {
  //     // "populate": [
  //     //   {
  //     //     "path": "payloads",
  //     //   },
  //     // ],
  //     "limit": 1,
  //   }
  // };
  // final jsonBody = convert.jsonEncode(json);

  final query = Options();
  // query.limit = 3;
  query.page = 3;
  query.select = [
    "water_landings",
    "serial",
    "type",
  ];

  var body = convert.jsonEncode(query.toJson());
  print(body);

  Response response = await api.queryCapsules(
    query: body,
  );

  if (response.statusCode == 200) {
    final jsonResponse = convert.jsonDecode(response.body);
    final page = PagenatedResponse.fromJson(jsonResponse);

    List<Capsule> caps =
        fromJsonList<Capsule>(page.docs, (e) => Capsule.fromJson(e));

    // print("id ${caps[caps.length - 1].id}");
    // print("serial ${caps[caps.length - 1].serial}");
    // print("water_landings ${caps[caps.length - 1].waterLandings}");
    // print("type ${caps[caps.length - 1].type}");
    print("length ${page.docs.length}");
    print("totalPages ${page.totalPages}");
    print("page ${page.page}");
    print("limit ${page.limit}");
  } else {
    print('Request failed with status: ${response.statusCode}.');
  }
}

Future<void> _fetchCompany<T>() async {
  final api = SpaceXApi();
  final response = await api.getCompany();

  if (response.statusCode == 200) {
    final jsonResponse = convert.jsonDecode(response.body);
    if (jsonResponse.runtimeType == List) {
      final obj = Company.fromJson(jsonResponse);
      print("obj ${obj.name}");
    } else {
      print('Error: tyep is differet ${jsonResponse.runtimeType}');
    }
  } else {
    print('Request failed with status: ${response.statusCode}.');
  }
}

Future<void> _fetchAllCapsules<T>() async {
  final api = SpaceXApi();
  final response = await api.getAllCapsules();

  if (response.statusCode == 200) {
    final jsonResponse = convert.jsonDecode(response.body);
    List<Capsule> caps =
        fromJsonList<Capsule>(jsonResponse, (e) => Capsule.fromJson(e));
    caps.forEach((element) {
      print("names ${element.serial}");
    });
  } else {
    print('Request failed with status: ${response.statusCode}.');
  }
}

fromJsonList<T>(List json, func) {
  List caps = json.map(func).cast<T>().toList();
  return caps;
}

Future<void> _fetchSingleCapsule() async {
  final api = SpaceXApi();
  final response = await api.getCapsuleById(id: "5e9e2c5cf359183bb73b266e");

  if (response.statusCode == 200) {
    final jsonResponse = convert.jsonDecode(response.body);
    Capsule caps = Capsule.fromJson(jsonResponse);
    print("serial ${caps.serial}");
  } else {
    print('Request failed with status: ${response.statusCode}.');
  }
}
